 ;---------------------------------------------------------------------
 ; title     : WLoader firmware
 ; file      : wloader.asm
 ; purpose   : 16f877 application loader
 ; info      : http://www.xs4all.nl/~wf/wouter/pic/wloader
 ;---------------------------------------------------------------------
 ; Copyright (C) 2000 Wouter van Ooijen
 ;
 ; This application is free software; you can redistribute it and/or
 ; modify it under the terms of the GNU Library General Public
 ; License as published by the Free Software Foundation; either
 ; version 2 of the License, or (at your option) any later version.
 ;
 ; This library is distributed in the hope that it will be useful,
 ; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ; Library General Public License for more details.
 ; 
 ; You should have received a copy of the GNU Library General Public
 ; License along with this library; if not, write to the
 ; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ; Boston, MA  02111-1307, USA.
 ;---------------------------------------------------------------------
 ;  6-SEP-2001   2.00    /dORIGIN, /dDEVICE_ID added, some corrections
 ;  5-SEP-2001   2.00    /dMAX232, /dDEBUG, /dNOAPP added
 ; 26-JUL-2001   1.04    no firmware changes
 ; 30-AUG-2000   1.03    data eeprom write corrected
 ; 17-AUG-2000   1.02    data eeprom write corrected
 ; 28-MAY-2000   1.01    baudrate configurable
 ; some other versions inbetween ....
 ; 20-MAR-2000   0.01    first version
 ;---------------------------------------------------------------------

 ;---------------------------------------------------------------------
 ; target configuration
 ; the PORTE, 2 can be replaced by another pin
 ; the frequency must be set according to the target
 ;---------------------------------------------------------------------

 #ifndef XTAL
    #define  XTAL D'20'*MHz
 #endif
 #ifndef PIN
    #define  PIN  porte,2
 #endif
 #ifdef MAX232
    #define  async_polarity active_high
 #else                     
 #ifdef ELCHEAPO
    #define  async_polarity active_low
 #else 
    errro "define either MAX232 or ELCHEAPO"
 #endif
 #endif
 #ifndef BAUDRATE
    #define BAUDRATE D'19200'
 #endif
 #ifndef ORIGIN
    #define ORIGIN H'1C00'
 #endif
 #ifndef DEVICE_ID
    #define DEVICE_ID B'00100110100000'
 #endif


 ;---------------------------------------------------------------------
 ; som necessary junk
 ;---------------------------------------------------------------------


 #define  baudrate BAUDRATE
 #define  xtal XTAL
 
 #include wovo877.inc
 
__xtal equ xtal
__baudrate equ baudrate
 if 0 == 1 ; show the xtal etc
    xtal       #v(__xtal)
    baudrate   #v(__baudrate)
 endif
 
 ;listing +200

 ;---------------------------------------------------------------------
 ; start of the loader image at 0
 ;---------------------------------------------------------------------

 org 0    
 movlw H'FF'
 movwf PCLATH
 goto main               
 sleep

 org ORIGIN                                                         
origin 

 ;---------------------------------------------------------------------
 ; used library files
 ;---------------------------------------------------------------------

 with     delay.inc
 with     convert.inc 
 with     async.inc
 with     port.inc


 ;---------------------------------------------------------------------
 ; I/O tools
 ;---------------------------------------------------------------------

 port_define PIN, async_polarity, in_out, async_pin
 port_create_tools

init_ports
 port_init
 return
 
 ;---------------------------------------------------------------------
 ; constants
 ;---------------------------------------------------------------------

#define mode_code  1
#define mode_data  2
#define mode_fuses 3


 ;---------------------------------------------------------------------
 ; global byte variables
 ;---------------------------------------------------------------------

 cblock
  current_mode, cmd 
  address_low,  address_high
  low_temp,     high_temp
  data0
  data1,        data2,        data3,        data4
  mask1,        mask2,        mask3,        mask4
  data1_temp,   data2_temp,   data3_temp,   data4_temp
  tmp, async_temp
 endc 


 ;---------------------------------------------------------------------
 ; global bit variables
 ;---------------------------------------------------------------------

 cblock
    flags
 endc
 #define active         flags, 0
 #define write_enable   flags, 1


 ;---------------------------------------------------------------------
 ; register bank selection macros
 ;---------------------------------------------------------------------

bank0 macro
 bcf STATUS, RP0
 bcf STATUS, RP1
 endm

bank1 macro
 bsf STATUS, RP0
 bcf STATUS, RP1
 endm

bank2 macro
 bcf STATUS, RP0
 bsf STATUS, RP1
 endm

bank3 macro
 bsf STATUS, RP0
 bsf STATUS, RP1
 endm


 ;---------------------------------------------------------------------
 ; debug macros
 ;---------------------------------------------------------------------

debug_send macro x
 movlw x
 call send_w
 endm

debug_send_hex macro _f
 movfw _f
 convert_w_hex_to_ascii
 call send_w
 endm

debug_send_hex2_routine
 convert_w_to_hl_hex 
 convert_w_hex_to_ascii
 call send_w
 return

debug_send_hex2 macro _f
 movfw _f
 call debug_send_hex2_routine
 endm

debug_send_data macro
 debug_send '('
 debug_send_hex data1
 debug_send_hex data2
 debug_send_hex data3
 debug_send_hex data4
 debug_send ')'
 endm

debug_send_address macro
 debug_send '('
 debug_send_hex2 address_high
 debug_send_hex2 address_low
 debug_send ':'
 bank2
 movfw low( EEADRH & H'7F' )
 bank0
 movwf tmp
 debug_send_hex2 tmp
 bank2
 movfw ( EEADR & H'7F' )
 bank0
 movwf tmp
 debug_send_hex2 tmp
 debug_send ')'
 endm


 ;---------------------------------------------------------------------
 ; macro: compare _file and _char, call _routine when equal
 ;---------------------------------------------------------------------

test_and_call macro _file, _char, _routine
 movfw _file
 sublw _char
 skpnz
  call _routine
 endm


 ;---------------------------------------------------------------------
 ; set the mask[1..4] bytes 
 ;---------------------------------------------------------------------

mask_for_14_bits
 move_const_via_w_to H'03', mask1
 move_const_via_w_to H'0F', mask2
 move_const_via_w_to H'0F', mask3
 move_const_via_w_to H'0F', mask4
 return

mask_for_8_bits
 move_const_via_w_to H'00', mask1
 move_const_via_w_to H'00', mask2
 move_const_via_w_to H'0F', mask3
 move_const_via_w_to H'0F', mask4
 return


 ;---------------------------------------------------------------------
 ; exchange data[1..4] with temp[1..4]
 ;---------------------------------------------------------------------

exchange_with_temp
 exchange_via_w data1, data1_temp
 exchange_via_w data2, data2_temp
 exchange_via_w data3, data3_temp
 exchange_via_w data4, data4_temp
 return


 ;---------------------------------------------------------------------
 ; address_low, address_high with their temp's
 ;---------------------------------------------------------------------

exchange_address_with_temp
 exchange_via_w address_high, high_temp
 exchange_via_w address_low, low_temp
 return


 ;---------------------------------------------------------------------
 ; set cmd to '?' when data[1..4] does not match temp[1..4]
 ;---------------------------------------------------------------------

compare_mask_mark macro _x, _y, _z
 movfw _x
 xorwf _y, w
 andwf _z, w
 movlw '?'
 skpz
  movwf cmd
 endm

compare_with_temp
 compare_mask_mark data1, data1_temp, mask1
 compare_mask_mark data2, data2_temp, mask2
 compare_mask_mark data3, data3_temp, mask3
 compare_mask_mark data4, data4_temp, mask4
 return


 ;---------------------------------------------------------------------
 ; set the eeprom address from a label
 ;---------------------------------------------------------------------

set_address_from_label macro label
 bank2
 move_const_via_w_to ( low label ), ( EEADR & H'7F' )
 move_const_via_w_to ( high label ), ( EEADRH & H'7F' )
 bank0
 endm


 ;---------------------------------------------------------------------
 ; set the address for read/write
 ;---------------------------------------------------------------------

set_address

 movf address_low, w
 bank2
 movwf ( EEADR & H'7F' )
 bank0

 movf address_high, w
 bank2
 movwf ( EEADRH & H'7F' )
 bank0

 return 


 ;---------------------------------------------------------------------
 ; get the read result
 ;---------------------------------------------------------------------

get_result

 bank2
 swapf ( EEDATH & H'7F' ), w
 bank0
 andlw H'0F'
 movwf data1

 bank2
 movfw ( EEDATH & H'7F' )
 bank0
 andlw H'0F'
 movwf data2
 
 bank2
 swapf ( EEDATA & H'7F' ), w
 bank0
 andlw H'0F'
 movwf data3

 bank2
 movfw ( EEDATA & H'7F' )
 bank0
 andlw H'0F'
 movwf data4
 
 return


 ;---------------------------------------------------------------------
 ; set the data for write
 ;---------------------------------------------------------------------

set_data

 bank0
 swapf data1, w
 iorwf data2, w
 bank2
 movwf ( EEDATH & H'7F' )

 bank0
 swapf data3, w
 iorwf data4, w
 bank2
 movwf ( EEDATA & H'7F' )
 bank0
 
 return


 ;---------------------------------------------------------------------
 ; the code and data read and write sequences according to Microchip
 ;---------------------------------------------------------------------

 ; code flash read sequence (16f877 datasheet p44)
read_flash
 bank3
 bsf ( EECON1 & H'7F' ), EEPGD
 bsf ( EECON1 & H'7F' ), RD
 nop
 nop
 bank0
 return

 ; code flash write sequence (16f877 datasheet p45)
write_flash
 bank3
 bsf ( EECON1 & H'7F' ), EEPGD
 bsf ( EECON1 & H'7F' ), WREN
 bcf ( INTCON & H'7F' ), GIE
 movlw H'55'
 movwf ( EECON2 & H'7F' )
 movlw H'AA'
 movwf ( EECON2 & H'7F' )
 bsf ( EECON1 & H'7F' ), WR
 nop
 nop
 bcf ( EECON1 & H'7F' ), WREN
 bank0

 return

 ; data eeprom read sequence (16f877 datasheet p43)
read_eeprom
 bank3
 bcf ( EECON1 & H'7F' ), EEPGD
 bsf ( EECON1 & H'7F' ), RD

 ; clear high byte
 bank2
 clrf ( EEDATH & H'7F' )

 bank0
 return

 ; data eeprom write sequence (16f877 datasheet p43)
 ; but busy loop instead of sleep
write_eeprom
 bank3
 bcf ( EECON1 & H'7F' ), EEPGD
 bsf ( EECON1 & H'7F' ), WREN
 bcf ( INTCON & H'7F' ), GIE
 movlw H'55'
 movwf ( EECON2 & H'7F' )
 movlw H'AA'
 movwf ( EECON2 & H'7F' )
 bsf ( EECON1 & H'7F' ), WR
 
write_eeprom_wait 
 btfsc ( EECON1 & H'7F' ), WR
  goto write_eeprom_wait

 bcf ( EECON1 & H'7F' ), WREN
 bank0
 return


 ;---------------------------------------------------------------------
 ; some simple commands
 ;---------------------------------------------------------------------

command_sleep
 bcf active
 return

command_increment
 incf address_low, f
 skpnz
  incf address_high, f
 return

command_next
 movfw data0
 convert_w_hex_to_ascii
 movwf cmd
 return

command_voltage
 ; not implemented, not needed in a 16f877 resident loader
 return
 
command_version
; goto relocate
 ; current version is 02.01
 move_const_via_w_to D'0', data1
 move_const_via_w_to D'2', data2
 move_const_via_w_to D'0', data3
 move_const_via_w_to D'1', data4
 return

command_name
 move_const_via_w_to inverse_hex_to_ascii('W'), data1
 move_const_via_w_to inverse_hex_to_ascii('L'), data2
 move_const_via_w_to inverse_hex_to_ascii('d'), data3
 move_const_via_w_to inverse_hex_to_ascii('r'), data4
 return


 ;---------------------------------------------------------------------
 ; device ID, activation etc.
 ;---------------------------------------------------------------------

my_id
 ; my ID is store in the next 3 locations
 nop
 nop
 nop
 nop

return_if_not_eq_const macro _f, _x
 movlw _x
 xorwf _f, w
 skpz
  return
 endm

wakeup_on_0
 return_if_not_eq_const data1, D'0'
 return_if_not_eq_const data2, D'0'
 return_if_not_eq_const data3, D'0'
 return_if_not_eq_const data4, D'0'
 bsf active
 return

return_if_not_eq macro _x, _y
 movfw _x
 endm

flash_read_and_compare macro _f
 call read_flash
 movfw _f
 bank2
 incf ( EEADR & H'7F' ), f
 skpnz
  incf ( EEADRH & H'7F' ), f
 xorwf ( EEDATA & H'7F' ), w
 bank0
 skpz
  return
 endm

wakeup_on_my_id
 set_address_from_label my_id
 flash_read_and_compare data1
 flash_read_and_compare data2
 flash_read_and_compare data3
 flash_read_and_compare data4
 bsf active
 return

command_wakeup
 bcf active
 call wakeup_on_0
 call wakeup_on_my_id
 return

flash_get_and_write macro _file
 movfw _file
 bank2
 movwf ( EEDATA & H'7F' )
 skpnz
  incf ( EEADRH & H'7F' ), f
 bank0
 call write_flash
 endm

command_burn
 set_address_from_label my_id
 flash_get_and_write data1
 flash_get_and_write data2
 flash_get_and_write data3
 flash_get_and_write data4
 return 

flash_read_and_store macro _file
 call read_flash
 bank2
 incf ( EEADR & H'7F' ), f
 skpnz
  incf ( EEADRH & H'7F' ), f
 movfw ( EEDATA & H'7F' )
 bank0
 movwf _file
 endm
 
command_query
 set_address_from_label my_id
 flash_read_and_store data1
 flash_read_and_store data2
 flash_read_and_store data3
 flash_read_and_store data4
 return 


 ;---------------------------------------------------------------------
 ; fake code and fuses addressing
 ;---------------------------------------------------------------------

command_passthrough
command_go
 movfw cmd
 call send_w
 delay_milliseconds 100
run_application
 bank1
 movlw 0
 movwf ( ADCON1 & H'7F' )
 bank0
 movlw 0
 movwf PCLATH
fake_address_0
 nop
fake_address_1
 nop
fake_address_2
 nop
 goto 3

 ; don't write beyond my origin
protect_myself
 bank2
 movfw ( EEADRH & H'7F' )
 sublw ( origin >> 8 ) - 1  
 bank0
 skpc
  bcf write_enable
 return

 ; set the fake address for 0..2
set_fake_0
 set_address_from_label fake_address_0
 return
set_fake_1
 set_address_from_label fake_address_1
 return
set_fake_2
 set_address_from_label fake_address_2
 return

 ; 0.. -> fake_address_0..
use_fake_address_0
 bank2
 movfw ( EEADRH & H'7F' )
 bank0
 skpz
  return
 bank2
 movfw ( EEADR & H'7F' )
 bank0
 addlw 0
 skpnz
  goto set_fake_0
 addlw -1 
 skpnz
  goto set_fake_1
 addlw -1
 skpnz
  goto set_fake_2
 return

 ; protect myself and replace the low 3 addresses
code_address_translation
 call protect_myself
 call use_fake_address_0
 bank0
; debug_send_address
 return

fake_device_id
 data DEVICE_ID

fake_fuses
 data CONFIG_VALUE

 ; never write to the fuses
 ; always read from the fake_fuses
fuses_address_translation     
 bank0
 bcf write_enable 
 
 bank2            
 movfw ( EEADR & H'7F' )
 bank0
 sublw 6
 skpz
  goto use_fuses   
  
 set_address_from_label fake_device_id
 return  
 
use_fuses 
 set_address_from_label fake_fuses
 return
 
 ; no translation needed for eeprom data
data_address_translation
 return

 ;---------------------------------------------------------------------
 ; program command(s)
 ;---------------------------------------------------------------------

command_program_code
 call mask_for_14_bits
 move_const_via_w_to mode_code, current_mode
 return

command_program_data
 call mask_for_8_bits
 move_const_via_w_to mode_data, current_mode
 return

command_program_fuses
 call mask_for_14_bits
 move_const_via_w_to mode_fuses, current_mode
 return

command_program_erase
 ; not implemented, not needed in a 16f877 resident loader
 return

command_program
 clrf address_high
 clrf address_low

 ; dispatch for data, code, fuses and erase
 test_and_call data4, H'0C', command_program_code
 test_and_call data4, H'0D', command_program_data
 test_and_call data4, H'0F', command_program_fuses
 test_and_call data4, H'0E', command_program_erase
 return


 ;---------------------------------------------------------------------
 ; read command(s)
 ;---------------------------------------------------------------------

read_data_logical
 call data_address_translation
 call read_eeprom
 return

read_code_logical
 call code_address_translation
 call read_flash
 return

read_fuses_logical
 call fuses_address_translation
 call read_flash
 return

 ; dispatch for data, code and fuses
command_read
 call set_address
 test_and_call current_mode, mode_data,  read_data_logical
 test_and_call current_mode, mode_code,  read_code_logical
 test_and_call current_mode, mode_fuses, read_fuses_logical
 call get_result
 return


 ;---------------------------------------------------------------------
 ; write command(s)
 ;---------------------------------------------------------------------

 ; translate, write only when allowed
 ; note: will always write....
write_data_logical
 call data_address_translation   
 bank0
 btfsc write_enable
  call write_eeprom
 return

 ; translate, write only when allowed
write_code_logical
 call code_address_translation
 btfsc write_enable
  call write_flash
 return

 ; translate, write only when allowed
 ; note: will never write....
write_fuses_logical
 call fuses_address_translation
 btfsc write_enable
  call write_flash
 return

command_write
 call set_address
 call set_data
 bsf write_enable

 ; dispatch for data, code and fuses
 test_and_call current_mode, mode_data,  write_data_logical
 test_and_call current_mode, mode_code,  write_code_logical
 test_and_call current_mode, mode_fuses, write_fuses_logical

 ; verify
 call exchange_with_temp
 call command_read
 call compare_with_temp
 return

command_lazy_write
 call exchange_with_temp
 call command_read
 call compare_with_temp
 movfw cmd
 xorlw '?'
 skpz
  return
 move_const_via_w_to 'L', cmd
 call exchange_with_temp
 call command_write
 return
 

 ;---------------------------------------------------------------------
 ; serial communication
 ;---------------------------------------------------------------------

send_w
 movwf async_temp 
 port_set_value async_pin, on
 port_set_direction async_pin, output
 movfw async_temp
 async_busy_send baudrate, 0, async_pin
 port_set_direction async_pin, input
 return

receive_w
 async_busy_receive baudrate, async_pin, 0
 skpc 
  goto receive_w
 return

 ;---------------------------------------------------------------------
 ; initialization
 ;---------------------------------------------------------------------

main

 ; disable A/D to be able to use port a
 bank1
 movlw 7
 movwf ( ADCON1 & H'7F' )
 bank0

#ifndef NOAPP                                                                                                                     
 if_port_value async_pin, off ; if line is mark
  goto run_application     
#endif

 call init_ports
 bsf active

#ifdef DEBUG
 movlw 'A'
 call send_w
 delay_milliseconds 100
 movlw 'B'
 call send_w
 delay_milliseconds 100
 movlw 'C'
 call send_w
 delay_milliseconds 100
#endif

 ;---------------------------------------------------------------------
 ; main loop
 ;---------------------------------------------------------------------

main_loop

 ; receive char and convert to uppercase
 call receive_w
 andlw H'7F'
 movwf cmd
 addlw 0 - 'a'
 skpnc
  bcf cmd, 5
  
 ; patch suggested by Wojciech Zabolotny to avoid
 ; sending the echo before the stopbit of the received char has ended
 delay_cycles (xtal/4)/baudrate 

 ; are we active?
 btfsc active 
  goto dispatch
 test_and_call cmd, 'H', command_wakeup
 goto update

dispatch
 test_and_call cmd, 'G', command_go
 test_and_call cmd, 'H', command_wakeup
 test_and_call cmd, 'I', command_increment
 test_and_call cmd, 'L', command_lazy_write
 test_and_call cmd, 'P', command_passthrough
 test_and_call cmd, 'Q', command_query
 test_and_call cmd, 'R', command_read
 test_and_call cmd, 'S', command_sleep
 test_and_call cmd, 'T', command_name
 test_and_call cmd, 'U', command_burn
 test_and_call cmd, 'V', command_version
 test_and_call cmd, 'W', command_write
 test_and_call cmd, 'X', command_program
 test_and_call cmd, 'Z', command_voltage
 test_and_call cmd, 'N', command_next ; must be last!

 ; update data0 .. data4
update
 move_from_via_w_to data1, data0
 move_from_via_w_to data2, data1
 move_from_via_w_to data3, data2
 move_from_via_w_to data4, data3
 movfw cmd
 convert_w_ascii_to_hex
 movwf data4

 ; when active echo the received (or modified) char
 movfw cmd
 btfsc active
  call send_w
 goto main_loop 

 end

